<?php
/**
* @version        $Id$
* @package        Joomla
* @subpackage    JFramework
* @copyright    Copyright (C) 2008-10 Omar Ramos, orware networks. All rights reserved.
* @license        GNU/GPL, see LICENSE.php
*/

// Check to ensure this file is included in Joomla!
defined( '_JEXEC' ) or die( 'Restricted access' );

jimport( 'joomla.plugin.plugin' );

/**
* Very Simple Image Rotator Content Plugin
*
* @package        Joomla
* @subpackage    JFramework
* @since 1.5
*/
class plgContentVSIR extends JPlugin
{
	private $cacheId;
	private $cacheRootPath;
	private $cachePath;
	private $cacheUrlPath;

	/**
	* Constructor
	*
	* For php4 compatability we must not use the __constructor as a constructor for plugins
	* because func_get_args ( void ) returns a copy of all passed arguments NOT references.
	* This causes problems with cross-referencing necessary for the observer design pattern.
	*
	* @param    object    $subject    The object to observe
	* @param    array    $config        An array that holds the plugin configuration
	* @since    1.5
	*/
	function plgContentVSIR( &$subject, $params )
	{
		parent::__construct( $subject, $params );

		// Collect default parameters so they can be reset later:
		$this->_defaults['include_subfolders'] = $this->params->get('include_subfolders');
		$this->_defaults['random'] = $this->params->get('random');
		$this->_defaults['use_direct_image_links'] = $this->params->get('use_direct_image_links');
		$this->_defaults['links_filename'] = $this->params->get('links_filename');
		$this->_defaults['links'] = $this->params->get('links');
		$this->_defaults['repeat'] = $this->params->get('repeat');
		$this->_defaults['alts'] = $this->params->get('alts');
		$this->_defaults['titles'] = $this->params->get('titles');
		$this->_defaults['target'] = $this->params->get('target');
		$this->_defaults['width'] = $this->params->get('width');
		$this->_defaults['height'] = $this->params->get('height');
		$this->_defaults['delay'] = $this->params->get('delay');
		$this->_defaults['trans'] = $this->params->get('trans');

		$this->_defaults['layout'] = $this->params->get('layout');
		$this->_defaults['center_images'] = $this->params->get('center_images');
		$this->_defaults['resize_images'] = $this->params->get('resize_images');
		$this->_defaults['resize_images_quality'] = $this->params->get('resize_images_quality');
		$this->_defaults['folder_permissions'] = $this->params->get('folder_permissions');
		$this->_defaults['limit'] = $this->params->get('limit');
	}

	public function onContentPrepare($context, &$row, &$params, $page = 0)
	{
		return $this->onPrepareContent($row, $params, $page);
	}

	/**
	* Example prepare content method
	*
	* Method is called by the view
	*
	* @param     object        The article object.  Note $article->text is also available
	* @param     object        The article params
	* @param     int            The 'page' number
	*/
	function onPrepareContent( &$article, &$params, $limitstart )
	{
		$mainframe = JFactory::getApplication();

		// simple performance check to determine whether bot should process further
		if (JString::strpos($article->text, 'vsir') === false) {
			return true;
		}

		// expression to search for
		$regex = '/{vsir\s*.*?}/i';

		// check whether plugin has been unpublished
		if (!$this->params->get('enabled', 1)) {
			$article->text = preg_replace($regex, '', $article->text);
			return true;
		}

		$matches = array();
		// find all instances of plugin and put in $matches
		preg_match_all($regex, $article->text, $matches);

		// Number of plugins
		$count = count($matches[0]);

		if ($count)
		{
			$plg_name = 'vsir';
			// load the plugin language file the proper way
			if($mainframe->isAdmin()){
				JPlugin::loadLanguage( 'plg_content_'.$plg_name );
			} else {
				JPlugin::loadLanguage( 'plg_content_'.$plg_name, 'administrator' );
			}

			// Add Stylesheet/JS
			//JHTML::stylesheet('vsir.css', 'plugins/content/vsir/vsir/assets/css/');
			JHtml::_('behavior.framework', true);
			JHTML::script('vsir.js', 'plugins/content/vsir/vsir/assets/js/');

			jimport('joomla.filesystem.folder');
		}

		// plugin only processes if there are any instances of the plugin in the text
		if ($count) {
			// Loop and collect data
			for ($i=0; $i < $count; $i++)
			{
				$vsirParameters = str_replace('vsir', '', $matches[0][$i]);
				$vsirParameters = str_replace('{', '', $vsirParameters);
				$vsirParameters = str_replace('}', '', $vsirParameters);
				$vsirParameters = trim($vsirParameters);

				// Initialize the output variable
				$output = '';

				$this->cacheId = md5($vsirParameters);

				// We want to make sure if caching has been enabled/disabled manually:
				$vsirParameters = $this->processOptionalCachingParameters($vsirParameters);

				if (empty($output))
				{
					$vsirParameters = $this->processOptionalParameters($vsirParameters);

					$this->params->set('folder', $vsirParameters);

					// Start creating the output string:
					$output = '';

					// Create Cache Folders if Needed:
					if ($this->params->get('resize_images', 0))
					{
						include_once dirname(__FILE__).'/vsir/classes/resize.php';

						// Create Folder Structure that will be needed later:
						$folders = JFolder::folders(JPATH_ROOT.'/'.$vsirParameters, '.', (bool) $this->params->get('include_subfolders', 0));

						$this->cacheRootPath = JPATH_ROOT.'/cache/plg_content_vsir/'.$this->cacheId.'/';
						$this->cachePath = $this->cacheRootPath.$vsirParameters.'/';
						JFolder::create($this->cachePath);
						$this->cacheUrlPath = 'cache/plg_content_vsir/'.$this->cacheId.'/';
						foreach($folders as $folderToCreate)
						{
							JFolder::create($this->cachePath . $folderToCreate, $this->params->get('folder_permissions', 0755));
						}
					}

					$folder	= $this->getFolder();
					$images	= $this->getImages($folder);

					$output .= $this->render($images);

				}

				$this->resetParameters();

				// Replace match with output string
				if (JString::strpos($article->text, '<p>'.$matches[0][$i].'</p>') !== false)
				{
					$article->text  = str_replace('<p>'.$matches[0][$i].'</p>', $output, $article->text);
				}
				else
				{
					$article->text  = str_replace($matches[0][$i], $output, $article->text);
				}

			}
		}
	}

	public function getImages($folder)
	{
		$images	= array();

		$links    = $this->params->get('links');
		$alts     = $this->params->get('alts');
		$titles   = $this->params->get('titles');

		if ($links)
		{
			$links  = preg_split("/[\n\r]+/", $links);
		}

		if ($alts)
		{
			$alts   = preg_split("/[\n\r]+/", $alts);
		}

		if ($titles)
		{
			$titles = preg_split("/[\n\r]+/", $titles);
		}

		$linksFilenamePath = JPATH_BASE.'/'.$folder.'/'.$this->params->get('links_filename', 'links.txt');

		$useCustomLinks = false;
		// Include Links File if it exists:
		if (is_file($linksFilenamePath))
		{
			$useCustomLinks = true;
			$data = JFile::read($linksFilenamePath);
			$dataArray = explode("\n", $data);
			$customLinksArray = array();
			$folder = str_replace( '\\', '/', $folder);
			foreach($dataArray as $customLinkData)
			{
				$parts = explode('|', trim($customLinkData));
				$customLinksArray[$folder.'/'.$parts[0]] = new stdClass;
				if (isset($parts[1]))
				{
					$customLinksArray[$folder.'/'.$parts[0]]->link = $parts[1];
				}
				if (isset($parts[2]))
				{
					$customLinksArray[$folder.'/'.$parts[0]]->title = $parts[2];
				}
				if (isset($parts[3]))
				{
					$customLinksArray[$folder.'/'.$parts[0]]->alt = $parts[3];
				}
			}
		}

		$target = $this->params->get('target', '_self');
		$repeat = $this->params->get('repeat', 0);
		$includeSubfolders = $this->params->get('include_subfolders', 0);
		$random = $this->params->get('random', 0);
		$width = (int)$this->params->get('width');
		$height = (int)$this->params->get('height');

		// check if directory exists
		if (is_dir(JPATH_BASE.'/'.$folder))
		{
			$files = $this->getFiles($folder, $includeSubfolders);
			$i = 0;
			$repeated = 0;

			foreach ($files as $file)
			{
				if ($this->isImage($file))
				{
					$file	    = str_replace( '\\', '/', $file );

					if ($this->params->get('resize_images', 0))
					{
						if (!JFile::exists($this->cacheRootPath.$file))
						{
							$resize = new Resize($file);
							$resize->resizeImage((int) $width, (int) $height);
							$resize->saveImage($this->cacheRootPath.$file, $this->params->get('resize_images_quality', 80));
						}
					}

					$alt        = '';
					$attribs    = array();
					$attribs['width'] = (int) $width;
					$attribs['height'] = (int) $height;
					if ($alts && isset($alts[$i]))
						$alt = $alts[$i];
					if ($titles && isset($titles[$i]))
						$attribs['title'] = $titles[$i];

					if ($this->params->get('resize_images', 0))
					{
						$images[$i] = JHTML::_('image', $this->cacheUrlPath.$file, $alt, $attribs);
					}
					else
					{
						$images[$i] = JHTML::_('image', $file, $alt, $attribs);
					}

					$imgTag = $images[$i];

					if ($this->params->set('use_direct_image_links', 1))
					{
						$images[$i] = JHTML::_('link', $file, $images[$i], ($target ? array('target' => '_blank') : '' ));
					}

					if ($useCustomLinks)
					{
						if (isset($customLinksArray[$file]))
						{
							$attribs['title'] = $customLinksArray[$file]->title;
							if (isset($customLinksArray[$file]->alt))
							{
								$alt = $customLinksArray[$file]->alt;
							}
							$images[$i] = JHTML::_('image', $file, $alt, $attribs);
							$images[$i] = JHTML::_('link', $customLinksArray[$file]->link, $images[$i], ($target ? array('target' => '_blank') : '' ));
						}
					}
					else
					{
						if ($links && (isset($links[$i]) or $repeat))
						{
							if (isset($links[$i]))
							{
								$images[$i] = JHTML::_('link', trim($links[$i]), $imgTag, ($target ? array('target' => '_blank') : '' ));
							}
							else
							{
								$repeated++;
								$images[$i] = JHTML::_('link', trim($links[$repeated-1]), $imgTag, ($target ? array('target' => '_blank') : '' ));
								if ($repeated == count($links)) $repeated = 0 ;
							}
						}
					}

					++$i;
				}
			}
		}

		if ($random)
		{
			shuffle($images);
			if ((int) $this->params->get('limit', 0))
			{
				$images = array_slice($images, 0, (int) $this->params->get('limit', 0));
			}
		}

		return $images;
	}

	public function getFolder($folderPath = '')
	{
		if (!empty($folderPath))
		{
			$folder = $folderPath;
		}
		else
		{
			$folder = $this->params->get('folder');
		}

		// Remove the trailing slash from the url (if any)
		if (substr($folder, -1) == '/') {
			$folder = substr($folder, 0 ,-1);
		}

		$LiveSite 	= JURI::base();

		// if folder includes livesite info, remove
		if ( JString::strpos($folder, $LiveSite) === 0 ) {
			$folder = str_replace( $LiveSite, '', $folder );
		}
		// if folder includes absolute path, remove
		if ( JString::strpos($folder, JPATH_SITE) === 0 ) {
			$folder= str_replace( JPATH_BASE, '', $folder );
		}
		$folder = str_replace('\\',DS,$folder);
		$folder = str_replace('/',DS,$folder);

		return $folder;
	}

	public function getFiles($folder, $includeSubfolders)
	{

		$dir      = JPATH_BASE.'/'.$folder;

		$files	  = array();
		$subfiles = array();

		if ($handle = opendir($dir))
		{
			while (false !== ($file = readdir($handle)))
			{
				if ($file != '.' && $file != '..' && $file != 'CVS' && $file != 'index.html' )
				{
					if (!is_dir($dir . DS . $file))
					{
						$files[] = $folder . DS . $file;
					}
					elseif ($includeSubfolders != 0)
					{
						$newfolder = $folder . DS . $file;
						$subfiles[]  = $this->getFiles($newfolder, $includeSubfolders, $this->params->get('random', 0));
					}
				}
			}
		}

		closedir($handle);

		sort($files);

		foreach ($subfiles as $subfile)
		{
			$files = array_merge($files,$subfile);
		}

		return $files;
	}

    public function isImage($file)
    {
		$file  = JFile::getName($file);
		$file  = JFile::getExt($file);
		$file  = strtolower($file);
        $types = array("jpg", "jpeg", "gif", "png");

        if (in_array($file, $types))
        {
			return true;
        }
        else
        {
			return false;
        }
    }

	public function buildCSS($uniqueInstanceId)
	{
		$width = $this->params->get('width', 'auto');
		$height = $this->params->get( 'height', 'auto');

		$centered = '';
		if ($this->params->get('center_images', 0))
		{
			$centered .= " margin-left: auto;";
			$centered .= " margin-right: auto;";
		}
		$style = "#vsir_" . $uniqueInstanceId . " {"
			. " position:relative;"
			. " height:" . $height . "px;"
			. " width:" . $width . "px;"
			. $centered
			. " }\n"
			. "#vsir_" . $uniqueInstanceId . " .vsig_slide_" . $uniqueInstanceId . " {"
			. " height:" . $height . "px;"
			. " position:absolute;"
			. " width:" . $width . "px;"
			. " }";

		return $style;
	}

	function resetParameters()
	{
		foreach($this->_defaults as $parameter => $value) {
			$this->params->set($parameter, $value);
		}
		$this->_getChildren = false;
	}

	function processOptionalParameters($text)
	{
		$text = str_replace('&nbsp;', ' ', $text);

		$setWidthPosition = JString::strpos($text, 'setWidth');

		if ($setWidthPosition !== false) {
			$findFirstSpaceAfter = JString::strpos($text, ' ', $setWidthPosition);
			$findSecondSpaceAfter = JString::strpos($text, ' ', $findFirstSpaceAfter+1);
			$width = (int) JString::trim(JString::substr($text, $findFirstSpaceAfter, ($findSecondSpaceAfter - $findFirstSpaceAfter)));
			$text = JString::substr_replace($text, '', $findFirstSpaceAfter, ($findSecondSpaceAfter - $findFirstSpaceAfter));
			$text = str_replace('setWidth', '', $text);
			$this->params->set('width', $width);
		}

		$setHeightPosition = JString::strpos($text, 'setHeight');

		if ($setHeightPosition !== false) {
			$findFirstSpaceAfter = JString::strpos($text, ' ', $setHeightPosition);
			$findSecondSpaceAfter = JString::strpos($text, ' ', $findFirstSpaceAfter+1);
			$height = (int) JString::trim(JString::substr($text, $findFirstSpaceAfter, ($findSecondSpaceAfter - $findFirstSpaceAfter)));
			$text = JString::substr_replace($text, '', $findFirstSpaceAfter, ($findSecondSpaceAfter - $findFirstSpaceAfter));
			$text = str_replace('setHeight', '', $text);
			$this->params->set('height', $height);
		}

		$setDelayPosition = JString::strpos($text, 'setDelay');

		if ($setDelayPosition !== false) {
			$findFirstSpaceAfter = JString::strpos($text, ' ', $setDelayPosition);
			$findSecondSpaceAfter = JString::strpos($text, ' ', $findFirstSpaceAfter+1);
			$delay = (int) JString::trim(JString::substr($text, $findFirstSpaceAfter, ($findSecondSpaceAfter - $findFirstSpaceAfter)));
			$text = JString::substr_replace($text, '', $findFirstSpaceAfter, ($findSecondSpaceAfter - $findFirstSpaceAfter));
			$text = str_replace('setDelay', '', $text);
			$this->params->set('delay', $delay);
		}

		$setTransitionDelayPosition = JString::strpos($text, 'setTransitionDelay');

		if ($setTransitionDelayPosition !== false) {
			$findFirstSpaceAfter = JString::strpos($text, ' ', $setTransitionDelayPosition);
			$findSecondSpaceAfter = JString::strpos($text, ' ', $findFirstSpaceAfter+1);
			$transitionDelay = (int) JString::trim(JString::substr($text, $findFirstSpaceAfter, ($findSecondSpaceAfter - $findFirstSpaceAfter)));
			$text = JString::substr_replace($text, '', $findFirstSpaceAfter, ($findSecondSpaceAfter - $findFirstSpaceAfter));
			$text = str_replace('setTransitionDelay', '', $text);
			$this->params->set('trans', $transitionDelay);
		}

		if (JString::strpos($text, 'randomizeImagesYes') !== false) {
			$text = str_replace('randomizeImagesYes', '', $text);
			$this->params->set('random', 1);
		}
		if (JString::strpos($text, 'randomizeImagesNo') !== false) {
			$text = str_replace('randomizeImagesNo', '', $text);
			$this->params->set('random', 0);
		}

		if (JString::strpos($text, 'includeSubfolders') !== false) {
			$text = str_replace('includeSubfolders', '', $text);
			$this->params->set('include_subfolders', 1);
		}
		if (JString::strpos($text, 'ignoreSubfolders') !== false) {
			$text = str_replace('ignoreSubfolders', '', $text);
			$this->params->set('include_subfolders', 0);
		}

		if (JString::strpos($text, 'directImageLinksYes') !== false) {
			$text = str_replace('directImageLinksYes', '', $text);
			$this->params->set('use_direct_image_links', 1);
		}
		if (JString::strpos($text, 'directImageLinksNo') !== false) {
			$text = str_replace('directImageLinksNo', '', $text);
			$this->params->set('use_direct_image_links', 0);
		}

		$setLinksFileNamePosition = JString::strpos($text, 'setLinksFileName');

		if ($setLinksFileNamePosition !== false) {
			$findFirstSpaceAfter = JString::strpos($text, ' ', $setLinksFileNamePosition);
			$findSecondSpaceAfter = JString::strpos($text, ' ', $findFirstSpaceAfter+1);
			$setLinksFileName = JString::trim(JString::substr($text, $findFirstSpaceAfter, ($findSecondSpaceAfter - $findFirstSpaceAfter)));
			$text = JString::substr_replace($text, '', $findFirstSpaceAfter, ($findSecondSpaceAfter - $findFirstSpaceAfter));
			$text = str_replace('setLinksFileName', '', $text);
			$this->params->set('links_filename', $setLinksFileName);
		}

		if (JString::strpos($text, 'resizeImagesOn') !== false) {
			$text = str_replace('resizeImagesOn', '', $text);
			$this->params->set('resize_images', 1);
		}
		if (JString::strpos($text, 'resizeImagesOff') !== false) {
			$text = str_replace('resizeImagesOff', '', $text);
			$this->params->set('resize_images', 0);
		}

		$setResizedImageQualityPosition = JString::strpos($text, 'setResizedImageQuality');

		if ($setResizedImageQualityPosition !== false) {
			$findFirstSpaceAfter = JString::strpos($text, ' ', $setResizedImageQualityPosition);
			$findSecondSpaceAfter = JString::strpos($text, ' ', $findFirstSpaceAfter+1);
			$setResizedImageQuality = (int) JString::trim(JString::substr($text, $findFirstSpaceAfter, ($findSecondSpaceAfter - $findFirstSpaceAfter)));
			$text = JString::substr_replace($text, '', $findFirstSpaceAfter, ($findSecondSpaceAfter - $findFirstSpaceAfter));
			$text = str_replace('setResizedImageQuality', '', $text);
			$this->params->set('resize_images_quality', $setResizedImageQuality);
		}

		$setFolderPermissionsPosition = JString::strpos($text, 'setFolderPermissions');

		if ($setFolderPermissionsPosition !== false) {
			$findFirstSpaceAfter = JString::strpos($text, ' ', $setFolderPermissionsPosition);
			$findSecondSpaceAfter = JString::strpos($text, ' ', $findFirstSpaceAfter+1);
			$setFolderPermissions = (int) JString::trim(JString::substr($text, $findFirstSpaceAfter, ($findSecondSpaceAfter - $findFirstSpaceAfter)));
			$text = JString::substr_replace($text, '', $findFirstSpaceAfter, ($findSecondSpaceAfter - $findFirstSpaceAfter));
			$text = str_replace('setFolderPermissions', '', $text);
			$this->params->set('folder_permissions', $setFolderPermissions);
		}

		$setRandomLimitPosition = JString::strpos($text, 'setRandomLimit');

		if ($setRandomLimitPosition !== false) {
			$findFirstSpaceAfter = JString::strpos($text, ' ', $setRandomLimitPosition);
			$findSecondSpaceAfter = JString::strpos($text, ' ', $findFirstSpaceAfter+1);
			$setRandomLimit = (int) JString::trim(JString::substr($text, $findFirstSpaceAfter, ($findSecondSpaceAfter - $findFirstSpaceAfter)));
			$text = JString::substr_replace($text, '', $findFirstSpaceAfter, ($findSecondSpaceAfter - $findFirstSpaceAfter));
			$text = str_replace('setRandomLimit', '', $text);
			$this->params->set('limit', $setRandomLimit);
		}

		if (JString::strpos($text, 'centerImagesOn') !== false) {
			$text = str_replace('centerImagesOn', '', $text);
			$this->params->set('center_images', 1);
		}
		if (JString::strpos($text, 'centerImagesOff') !== false) {
			$text = str_replace('centerImagesOff', '', $text);
			$this->params->set('center_images', 0);
		}

		return trim($text);
	}

	function processOptionalCachingParameters($text)
	{
		if (JString::strpos($text, 'forceCachingOn') !== false) {
			$text = str_replace('forceCachingOn', '', $text);
			$this->params->set('cache', 1);
		}
		if (JString::strpos($text, 'forceCachingOff') !== false) {
			$text = str_replace('forceCachingOff', '', $text);
			$this->params->set('cache', 0);
		}
		if (JString::strpos($text, 'showTooltip') !== false || ((int) $this->params->get('description_tooltip', 0))) {
			JHTML::_('behavior.tooltip');
		}

		return trim($text);
	}

	function render($images)
	{
		$output = '';

		$uniqueInstanceId = rand(1, 999999);

		// Get module path
		$layout = $this->params->get('layout', 'default');
		$path = JPATH_BASE.'/plugins/content/vsir/vsir/layouts/'.$layout.'.php';

		// Load the module
		if (file_exists($path))
		{
			$content = '';
			ob_start();
			include $path;
			$output = ob_get_contents().$content;
			ob_end_clean();
		}

		return $output;
	}
}